# -*- coding: utf-8 -*-
from collective.feedback.interfaces import ICollectiveFeedbackStore
from collective.feedback.testing import RESTAPI_TESTING
from datetime import datetime
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from souper.soup import get_soup
from souper.soup import Record
from zope.component import getUtility

import transaction
import unittest


class TestCustomerSatisfactionGet(unittest.TestCase):
    layer = RESTAPI_TESTING

    def add_record(self, date=None, vote="", uid="", comment="", title=""):
        if not date:
            date = datetime.now()
        soup = get_soup("feedback_soup", self.portal)
        transaction.commit()
        record = Record()
        record.attrs["vote"] = vote
        record.attrs["date"] = date

        if comment:
            record.attrs["comment"] = comment
        if uid:
            record.attrs["uid"] = uid
        if title:
            record.attrs["title"] = title
        soup.add(record)
        transaction.commit()

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.url = "{}/@feedback".format(self.portal_url)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.document1 = api.content.create(
            title="document 1", container=self.portal, type="Document"
        )
        self.document2 = api.content.create(
            title="document 1", container=self.portal, type="Document"
        )
        transaction.commit()

        # add some reviews
        tool = getUtility(ICollectiveFeedbackStore)
        tool.add(
            {
                "vote": 1,
                "uid": self.document1.UID(),
                "title": self.document1.title,
            }
        )
        tool.add(
            {
                "vote": 2,
                "uid": self.document1.UID(),
                "title": self.document1.title,
            }
        )
        tool.add(
            {
                "vote": 3,
                "uid": self.document2.UID(),
                "title": self.document2.title,
            }
        )

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        transaction.commit()

    def tearDown(self):
        self.api_session.close()
        soup = get_soup("feedback_soup", self.portal)
        soup.clear()

    def test_deleting_a_content_does_not_remove_entries(self):
        response = self.api_session.get(self.url)
        res = response.json()
        self.assertEqual(res["items_total"], 2)

        api.content.delete(obj=self.document1)
        transaction.commit()

        response = self.api_session.get(self.url)
        res = response.json()
        self.assertEqual(res["items_total"], 2)



# -*- coding: utf-8 -*-
from collective.feedback.interfaces import ICollectiveFeedbackStore
from collective.feedback.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from zope.component import getUtility

import transaction
import unittest


class TestAdd(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        api.user.create(
            email="memberuser@example.com",
            username="memberuser",
            password="secret!!",
        )

        self.document = api.content.create(
            title="Document", container=self.portal, type="Document"
        )
        api.content.transition(obj=self.document, transition="publish")

        self.private_document = api.content.create(
            title="restricted document", container=self.portal, type="Document"
        )
        transaction.commit()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.anon_api_session = RelativeSession(self.portal_url)
        self.anon_api_session.headers.update({"Accept": "application/json"})

        self.url = "{}/@feedback-add".format(self.document.absolute_url())
        self.url_private_document = "{}/@feedback-add".format(
            self.private_document.absolute_url()
        )

    def tearDown(self):
        self.api_session.close()
        self.anon_api_session.close()

    def test_required_params(self):
        """ """
        # vote is required
        res = self.anon_api_session.post(self.url, json={"honey": ""})
        self.assertEqual(res.status_code, 400)
        self.assertEqual(res.json()["message"], "Campo obbligatorio mancante: vote")

    def test_correctly_save_data(self):
        self.anon_api_session.post(
            self.url,
            json={"vote": 3, "comment": "i disagree", "honey": ""},
        )
        transaction.commit()
        tool = getUtility(ICollectiveFeedbackStore)
        self.assertEqual(len(tool.search()), 1)

        # Anonymous cannot vote without access to document
        self.anon_api_session.post(
            self.url_private_document,
            json={"vote": 2, "comment": "i disagree", "honey": ""},
        )
        transaction.commit()
        # Number of results did not increase, cause user is unauthorized to vote
        self.assertEqual(len(tool.search()), 1)

    def test_store_only_known_fields(self):
        self.anon_api_session.post(
            self.url,
            json={
                "vote": "nok",
                "comment": "i disagree",
                "unknown": "mistery",
                "honey": "",
            },
        )
        transaction.commit()
        tool = getUtility(ICollectiveFeedbackStore)
        res = tool.search()
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0]._attrs.get("unknown", None), None)
        self.assertEqual(res[0]._attrs.get("vote", None), "nok")
        self.assertEqual(res[0]._attrs.get("comment", None), "i disagree")

    def test_honeypot_is_required(self):
        res = self.anon_api_session.post(self.url, json={})
        self.assertEqual(res.status_code, 403)

        res = self.anon_api_session.post(self.url, json={"vote": "ok"})
        self.assertEqual(res.status_code, 403)

        # HONEYPOT_FIELD is set in testing.py

        res = self.anon_api_session.post(self.url, json={"vote": "ok", "honey": ""})
        self.assertEqual(res.status_code, 204)

        # this is compiled by a bot
        res = self.anon_api_session.post(
            self.url, json={"vote": "ok", "honey": "i'm a bot"}
        )
        self.assertEqual(res.status_code, 403)



from AccessControl import Unauthorized
from collective.feedback.interfaces import ICollectiveFeedbackStore
from copy import deepcopy
from datetime import datetime
from plone import api
from plone.restapi.batching import HypermediaBatch
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.services import Service
from six import StringIO
from zope.component import getUtility
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse

import csv


@implementer(IPublishTraverse)
class FeedbackGet(Service):
    """Service for getting feedbacks"""

    def __init__(self, context, request):
        super().__init__(context, request)
        self.params = []

    def publishTraverse(self, request, uid):
        # Consume any path segments after /@users as parameters
        self.params.append(uid)
        return self

    def reply(self):
        if self.params:
            # single object detail
            results = self.get_single_object_feedbacks(self.params[0])
        else:
            results = self.get_data()
        batch = HypermediaBatch(self.request, results)
        data = {
            "@id": batch.canonical_url,
            "items": [self.fix_fields(data=x) for x in batch],
            "items_total": batch.items_total,
        }
        links = batch.links
        if links:
            data["batching"] = links

        data["actions"] = {"can_delete_feedbacks": self.can_delete_feedbacks()}
        return data

    def can_delete_feedbacks(self):
        return api.user.has_permission("collective.feedback: Delete Feedbacks")

    def fix_fields(self, data):
        """
        Make data json compatible
        """
        for k, v in data.items():
            data[k] = json_compatible(v)
        return data

    def parse_query(self):
        query = deepcopy(self.request.form)
        query = unflatten_dotted_dict(query)
        res = {}
        if "sort_on" in query:
            res["sort_index"] = query["sort_on"]
            del query["sort_on"]
        if "sort_order" in query:
            order = query["sort_order"]
            reverse = True
            if order in ["asc", "ascending"]:
                reverse = False
            res["reverse"] = reverse
            del query["sort_order"]
        res["query"] = query
        return res

    def get_commented_obj(self, uid):
        """
        Return obj based on uid.
        """
        try:
            obj = api.content.get(UID=uid)
        except Unauthorized:
            return

        if not obj:
            return

        if not api.user.has_permission(
            "collective.feedback: Access Feedbacks", obj=obj
        ):
            # user does not have that permission on object
            return

        return obj

    def get_single_object_feedbacks(self, uid):
        """
        Return data for single object
        """
        commented_object = self.get_commented_obj(uid=uid)
        if not commented_object:
            return []
        tool = getUtility(ICollectiveFeedbackStore)
        results = tool.search(query={"uid": uid})
        feedbacks = []

        for record in results:
            feedbacks.append(
                {
                    "uid": uid,
                    "date": record._attrs.get("date", ""),
                    "vote": record._attrs.get("vote", ""),
                    "answer": record._attrs.get("answer", ""),
                    "comment": record._attrs.get("comment", ""),
                    "title": commented_object.title,
                }
            )

        return feedbacks

    def get_data(self):
        tool = getUtility(ICollectiveFeedbackStore)
        feedbacks = {}

        query = unflatten_dotted_dict(self.request.form)
        text = query.get("title", "")
        if text:
            query_res = tool.search(query={"title": text})
        else:
            query_res = tool.search()

        for feedback in query_res:
            uid = feedback._attrs.get("uid", "")
            date = feedback._attrs.get("date", "")
            vote = feedback._attrs.get("vote", "")
            if uid not in feedbacks:
                obj = self.get_commented_obj(uid=uid)
                if not obj and not api.user.has_permission(
                    "collective.feedback: Show Deleted Feedbacks"
                ):
                    # only manager can list deleted object's reviews
                    continue
                new_data = {
                    "vote_num": 0,
                    "vote_sum": 0,
                    "comments": 0,
                    "title": feedback._attrs.get("title", ""),
                    "uid": uid,
                }

                if obj:
                    new_data["title"] = obj.Title()
                    new_data["url"] = obj.absolute_url()

                feedbacks[uid] = new_data

            # vote avg
            data = feedbacks[uid]
            data["vote_num"] += 1
            data["vote_sum"] += vote

            # number of comment
            comment = feedback._attrs.get("comment", "")
            answer = feedback._attrs.get("answer", "")
            if comment or answer:
                data["comments"] += 1

            # last date comment
            if not data.get("last_vote", None):
                data["last_vote"] = date
            else:
                if data["last_vote"] < date:
                    data["last_vote"] = date

        # avg calculation
        for uid, feedback in feedbacks.items():
            feedback["vote"] = feedback.pop("vote_sum") / feedback.pop("vote_num")

        result = list(feedbacks.values())

        # sort
        sort_on = self.request.form.get("sort_on", "last_vote")
        sort_order = self.request.form.get("sort_order", "desc")
        reverse = sort_order.lower() in ["desc", "descending", "reverse"]
        if sort_on in ["vote", "title", "last_vote", "comments"]:
            result = sorted(result, key=lambda k: k[sort_on], reverse=reverse)

        return result


class FeedbackGetCSV(FeedbackGet):
    """Service for getting feedbacks as csv file"""

    type = "feedbacks"

    def render(self):
        data = self.get_data()
        if isinstance(data, dict):
            if data.get("error", False):
                self.request.response.setStatus(500)
                return dict(
                    error=dict(
                        type="InternalServerError",
                        message="Unable export. Contact site manager.",
                    )
                )
        self.request.response.setHeader("Content-Type", "text/comma-separated-values")
        now = datetime.now()
        self.request.response.setHeader(
            "Content-Disposition",
            f'attachment; filename="{self.type}_{now.strftime("%d%m%Y-%H%M%S")}.csv"',
        )
        self.request.response.write(data)

    def get_data(self):
        tool = getUtility(ICollectiveFeedbackStore)
        sbuf = StringIO()
        rows = []
        columns = ["title", "url", "vote", "comment", "date", "answer"]

        for item in tool.search():
            uid = item._attrs.get("uid", "")
            obj = self.get_commented_obj(uid=uid)
            if not obj:
                continue

            data = {}
            for k, v in item.attrs.items():
                if k not in columns:
                    continue

                if isinstance(v, list):
                    v = ", ".join(v)

                if isinstance(v, int):
                    v = str(v)

                val = json_compatible(v)
                data[k] = val

            data["url"] = obj.absolute_url()
            rows.append(data)

        writer = csv.DictWriter(sbuf, fieldnames=columns, delimiter=",")
        writer.writeheader()
        for row in rows:
            try:
                writer.writerow(row)
            except Exception:
                return {"error": True}

        res = sbuf.getvalue()
        sbuf.close()

        return res.encode()



# -*- coding: utf-8 -*-
from collective.feedback.interfaces import ICollectiveFeedbackStore
from collective.feedback.testing import FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getUtility

import transaction
import unittest


class TestTool(unittest.TestCase):
    layer = FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def tearDown(self):
        tool = getUtility(ICollectiveFeedbackStore)
        tool.clear()
        transaction.commit()

    def test_correctly_add_data(self):
        tool = getUtility(ICollectiveFeedbackStore)
        self.assertEqual(len(tool.search()), 0)
        tool.add({"vote": 4})
        self.assertEqual(len(tool.search()), 1)

    def test_only_store_defined_fields(self):
        tool = getUtility(ICollectiveFeedbackStore)
        self.assertEqual(len(tool.search()), 0)
        id = tool.add(
            {
                "vote": 4,
                "foo": "foo",
                "unknown": "mistery",
                "title": "title",
                "comment": "comment",
            }
        )
        self.assertEqual(len(tool.search()), 1)

        item = tool.get_record(id)
        self.assertEqual(item.attrs.get("unknown", None), None)
        self.assertEqual(item.attrs.get("foo", None), None)
        self.assertEqual(item.attrs.get("vote", None), 4)
        self.assertEqual(item.attrs.get("title", None), "title")
        self.assertEqual(item.attrs.get("comment", None), "comment")

    def test_update_record(self):
        tool = getUtility(ICollectiveFeedbackStore)
        id = tool.add({"vote": 4, "comment": "is ok"})

        item = tool.get_record(id)
        self.assertEqual(item.attrs.get("vote", None), 4)
        self.assertEqual(item.attrs.get("comment", None), "is ok")

        tool.update(id=id, data={"vote": 2, "comment": "not ok"})
        item = tool.get_record(id)
        self.assertEqual(item.attrs.get("vote", None), 2)
        self.assertEqual(item.attrs.get("comment", None), "not ok")

    def test_update_record_return_error_if_id_not_found(self):
        tool = getUtility(ICollectiveFeedbackStore)
        res = tool.update(id=1222, data={})
        self.assertEqual(res, {"error": "NotFound"})

    def test_delete_record(self):
        tool = getUtility(ICollectiveFeedbackStore)
        foo = tool.add({"vote": 1, "comment": "is ok", "uid": 1234})
        tool.add({"vote": 1, "comment": "is ok", "uid": 5678})

        self.assertEqual(len(tool.search()), 2)
        tool.delete(id=foo)
        self.assertEqual(len(tool.search()), 1)

    def test_delete_record_return_error_if_id_not_found(self):
        tool = getUtility(ICollectiveFeedbackStore)
        res = tool.delete(id=1222)
        self.assertEqual(res, {"error": "NotFound"})

    def test_clear(self):
        tool = getUtility(ICollectiveFeedbackStore)
        tool.add({"vote": 1, "comment": "is ok", "uid": 1234})
        tool.add({"vote": 1, "comment": "is ok", "uid": 5678})
        transaction.commit()

        self.assertEqual(len(tool.search()), 2)

        tool.clear()
        self.assertEqual(len(tool.search()), 0)



from collective.feedback.interfaces import ICollectiveFeedbackStore
from plone.protect.interfaces import IDisableCSRFProtection
from plone.restapi.services import Service
from zExceptions import BadRequest
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse


@implementer(IPublishTraverse)
class FeedbackDelete(Service):
    """Service for delete feedbacks"""

    def __init__(self, context, request):
        super().__init__(context, request)
        self.params = []

    def publishTraverse(self, request, name):
        # Consume any path segments after /@users as parameters
        self.params.append(name)
        return self

    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        tool = getUtility(ICollectiveFeedbackStore)

        if self.params:
            try:
                self.id = self.params[0]
            except ValueError:
                raise BadRequest("Id should be a number.")

            feedbacks = tool.search(query={"uid": self.id})
            for feedback in feedbacks:
                res = tool.delete(id=feedback.intid)
                if not res:
                    continue
                if res.get("error", "") == "NotFound":
                    raise BadRequest('Unable to find item with id "{}"'.format(self.id))
                self.request.response.setStatus(500)
                return dict(
                    error=dict(
                        type="InternalServerError",
                        message="Unable to delete item. Contact site manager.",
                    )
                )

        else:
            tool.clear()

        return self.reply_no_content()



# -*- coding: utf-8 -*-
from collective.feedback.interfaces import ICollectiveFeedbackStore
from collective.feedback.testing import RESTAPI_TESTING
from datetime import datetime
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.testing import RelativeSession
from souper.soup import get_soup
from souper.soup import Record
from zope.component import getUtility

import transaction
import unittest


class TestGet(unittest.TestCase):
    layer = RESTAPI_TESTING

    def add_record(self, date=None, vote="", uid="", comment="", title=""):
        if not date:
            date = datetime.now()
        soup = get_soup("feedback_soup", self.portal)
        transaction.commit()
        record = Record()
        record.attrs["vote"] = vote
        record.attrs["date"] = date

        if comment:
            record.attrs["comment"] = comment
        if uid:
            record.attrs["uid"] = uid
        if title:
            record.attrs["title"] = title
        soup.add(record)
        transaction.commit()

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.url = "{}/@feedback".format(self.portal_url)
        self.tool = getUtility(ICollectiveFeedbackStore)

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        api.user.create(
            email="member@example.com",
            username="member",
            password="secret!!",
        )
        api.user.create(
            email="global@example.com",
            username="global",
            password="secret!!",
        )

        api.user.create(
            email="local@example.com",
            username="local",
            password="secret!!",
        )
        # create some contents
        self.document = api.content.create(
            title="document", container=self.portal, type="Document"
        )
        api.content.transition(obj=self.document, transition="publish")

        self.restricted_document = api.content.create(
            title="restricted document", container=self.portal, type="Document"
        )

        transaction.commit()

        api.user.grant_roles(
            username="global",
            roles=["Editor"],
        )
        api.user.grant_roles(
            username="local", roles=["Editor"], obj=self.restricted_document
        )

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        transaction.commit()

    def tearDown(self):
        self.api_session.close()
        self.tool.clear()

    def test_anon_cant_access_endpoint(self):
        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})
        self.assertEqual(api_session.get(self.url).status_code, 401)

    def test_admin_can_access_endpoint(self):
        self.assertEqual(self.api_session.get(self.url).status_code, 200)

    def test_other_users_can_access_endpoint(self):
        for username in ["member", "local", "global"]:
            api_session = RelativeSession(self.portal_url)
            api_session.headers.update({"Accept": "application/json"})
            api_session.auth = (username, "secret!!")
            self.assertEqual(api_session.get(self.url).status_code, 200)

    def test_endpoint_returns_data(self):
        response = self.api_session.get(self.url)
        res = response.json()
        self.assertEqual(res["items_total"], 0)
        now = datetime.now()
        self.add_record(vote=1, comment="is ok", date=now)

        response = self.api_session.get(self.url)
        res = response.json()

        self.assertEqual(res["items_total"], 1)
        self.assertEqual(
            res["items"],
            [
                {
                    "comments": 1,
                    "last_vote": json_compatible(now),
                    "title": "",
                    "uid": "",
                    "vote": 1.0,
                }
            ],
        )

    def test_basic_user_cant_see_data(self):
        now = datetime.now()
        self.add_record(
            vote=1, date=now, comment="is ok for member", uid=self.document.UID()
        )
        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})
        api_session.auth = ("member", "secret!!")

        response = api_session.get(self.url)
        res = response.json()

        self.assertEqual(res["items_total"], 0)

    def test_global_editor_can_see_all_data(self):
        now = datetime.now()
        self.add_record(
            vote=1, date=now, comment="is ok for global", uid=self.document.UID()
        )
        self.add_record(
            vote=1,
            date=now,
            comment="ok also for restricted",
            uid=self.restricted_document.UID(),
        )
        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})
        api_session.auth = ("global", "secret!!")

        response = api_session.get(self.url)
        res = response.json()

        self.assertEqual(res["items_total"], 2)

    def test_local_editor_can_see_only_data_for_his_contents(self):
        now = datetime.now()
        self.add_record(
            vote=1, date=now, comment="is ok for global", uid=self.document.UID()
        )
        self.add_record(
            vote=1,
            date=now,
            comment="ok also for restricted",
            uid=self.restricted_document.UID(),
        )
        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})
        api_session.auth = ("local", "secret!!")

        response = api_session.get(self.url)
        res = response.json()

        self.assertEqual(res["items_total"], 1)
        self.assertEqual(res["items"][0]["uid"], self.restricted_document.UID())

    def test_only_admins_can_see_deleted_contents(self):
        now = datetime.now()
        self.add_record(vote=1, date=now, comment="is ok", uid=self.document.UID())
        self.add_record(
            vote=1,
            date=now,
            comment="ok for deleted content",
            uid="qwertyuiop",
        )

        response = self.api_session.get(self.url)
        res = response.json()
        self.assertEqual(res["items_total"], 2)

        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})
        api_session.auth = ("global", "secret!!")

        response = api_session.get(self.url)
        res = response.json()

        self.assertEqual(res["items_total"], 1)

    def test_actions_list_returned(self):
        response = self.api_session.get(self.url)
        res = response.json()
        self.assertIn("actions", res)

    def test_users_with_permission_have_can_delete_feedbacks_action(self):
        response = self.api_session.get(self.url)
        res = response.json()
        self.assertIn("can_delete_feedbacks", res["actions"])
        self.assertTrue(res["actions"]["can_delete_feedbacks"])

    def test_users_without_permission_dont_have_can_delete_feedbacks_action(self):
        api_session = RelativeSession(self.portal_url)
        api_session.headers.update({"Accept": "application/json"})
        api_session.auth = ("global", "secret!!")

        response = api_session.get(self.url)
        res = response.json()

        self.assertIn("can_delete_feedbacks", res["actions"])
        self.assertFalse(res["actions"]["can_delete_feedbacks"])



# -*- coding: utf-8 -*-
from plone import api
from plone.app.upgrade.utils import installOrReinstallProduct

import logging


logger = logging.getLogger(__name__)

DEFAULT_PROFILE = "profile-collective.feedback:default"


def update_profile(context, profile, run_dependencies=True):
    context.runImportStepFromProfile(DEFAULT_PROFILE, profile, run_dependencies)


def update_types(context):
    update_profile(context, "typeinfo")


def update_rolemap(context):
    update_profile(context, "rolemap")


def update_registry(context):
    update_profile(context, "plone.app.registry", run_dependencies=False)


def update_catalog(context):
    update_profile(context, "catalog")


def update_controlpanel(context):
    update_profile(context, "controlpanel")


def update_actions(context):
    update_profile(context, "actions")


def to_1100(context):
    installOrReinstallProduct(api.portal.get(), "souper.plone")
    context.runAllImportStepsFromProfile("profile-collective.feedback:to_1100")

    # remove broken action
    if "feedback-dashboard" in context.portal_actions.user:
        del context.portal_actions.user["feedback-dashboard"]
